﻿:Namespace CIA
    WINDOWS←#.Util.InitCauseway ⎕THIS

    ∇ LX;sp
      Update 0
      #.Util.View 1 1 Scatter'POPULATION' 'GDP'
      #.Util.View 0 0 Bubble'GDP_PER_CAPITA' 'LIFE_EXPECTANCY' 'HEALTH_EXPENDITURES'
      #.Util.View(1 1)(10000 0.5)100 4 Contour'GDP' 'POPULATION'('GDP_PER_CAPITA' 'MILITARY_EXPENDITURES')
      #.Util.View(1 1)(2 2 10 100)100 4 Contour'GDP_PER_CAPITA' 'HEALTH_EXPENDITURES'('UNDERWEIGHT_CHILDREN' 'OBESITY_ADULT' 'INFANT_MORTALITY' 'MATERNAL_MORTALITY')
    ∇

    ∇ (variable title info unit countries values)←ReadFile file;tie;content;head;Part3;vfi;ERR;mask
      Part3←{⎕ML←3 ⋄ ⍺⊂[⍺⍺]⍵} ⋄ ERR←'ERR'  ⍝ make ⎕SIGNAL DOMAIN ERROR rather than pop up the stack
      tie←file ⎕NTIE 0
      content←'UTF-8'⎕UCS ⎕UCS ⎕NREAD tie 80(⎕NSIZE tie)0
      ⎕NUNTIE tie
      title←('rawdata_[0-9][0-9][0-9][0-9]_(.+)\.txt'⎕S'\1')file
      variable←⊃('.*'⎕R'\u&')title       ⍝ all upper case with underscores
      title←⊃('^.'⎕R'\u&')('_'⎕R' ')title ⍝ upper case first letter and replace underscores by spaces
      ⎕SIGNAL(0∊⍴title)/ERR
      content←('\r\n'⎕R'\n'⍠'Mode' 'D')content   ⍝ replace (CR+LF) with single newline
      content←(~content∊⎕UCS 13 10)(1 Part3)content  ⍝ split at newlines
      head←'COUNTRY COMPARISON :: ' ⋄ ⎕SIGNAL(head≢(⍴head)↑1⊃content)/ERR  ⍝ check that first line starts as expected
      info←2⊃content ⋄ unit←3⊃content ⋄ content←3↓content ⍝ extract header info (first line is just a repeat of the start of second)
      content←↑content ⋄ content←(⌽~∧\⌽' '∧.=content)/content  ⍝ make matrix and trim trailing blank columns
      content←(~<\' '∧.=content)(2 Part3)content←↑content  ⍝ split at first blank column
      ⎕SIGNAL((2≠2⊃⍴content)∨(↑content[;1])≢(↑⍕¨⍳≢content))/ERR ⋄ content←↑content[;2]  ⍝ ignore first column which is ranking
      content←(~∧\' '∧.=content)/content  ⍝ trim leading blank columns
      content←(⌽~<\⌽' '∧.=content)(2 Part3)content  ⍝ split at last blank column
      ⎕SIGNAL(2≠2⊃⍴content)/ERR              ⍝ should have two columns
      values←{(+/∨\' '≠⌽⍵)↑¨↓⍵}↑content[;2]   ⍝ values are last column
      countries←{(+/∨\' '≠⌽⍵)↑¨↓⍵}↑content[;1] ⍝ countries are all but last column
      :If unit≡,'$' ⋄ values←values~¨'$' ⋄ :EndIf  ⍝ $ is usually repeated in all numbers
      values←values~¨','  ⍝ comma useful only for humans
      ((values='-')/¨values)←'¯'  ⍝ APL is correct but not standard
      vfi←⎕VFI¨values
      :If ∨/mask←(,1)∘≢¨⊃¨vfi ⍝ failed to convert some numbers
          ⎕←'Incorrect numbers : file "',file,'", lines ',(⍕mask/⍳⍴mask),' : ',⍕mask/values
          vfi←(~mask)/vfi  ⍝ trim them
          countries←(~mask)/countries
      :EndIf
      values←⊃¨2⊃¨vfi
      ⎕SIGNAL(≠/⊃∘⍴¨values countries)/ERR
    ∇

    ∇ Update force;files
      :If force∨0=⎕NC'DATA'
          :If WINDOWS
              files←'CIA\'∘,¨⎕CMD'DIR /B CIA\*.txt'
          :Else
              files←⎕SH'ls CIA/*.txt'
          :EndIf
          DATA←↑ReadFile¨files
          (ID TITLE INFO UNIT COUNTRY VALUE)←⍳2⊃⍴DATA  ⍝ column names
          COUNTRIES←{⍵[⍋↑⍵]}∪⊃,/DATA[;COUNTRY]
      :EndIf
    ∇

    ∇ results←Search keyword;rows
      :If 1<|≡keyword ⋄ results←⊃⍪/Search¨keyword ⋄ :Return ⋄ :EndIf
      rows←⎕IO+∪(keyword ⎕S 2⍠'IC' 1)DATA[;INFO]
      results←DATA[rows;ID TITLE UNIT INFO]
    ∇

    EIS←{1=≡⍵:⊂⍵ ⋄ ⍵}  ⍝ ⊆ in v16.0
    MISSVAL←¯1e300  ⍝ missing value (should not appear in regular data)

    ∇ rows←GetRows ids;mask
      rows←DATA[;ID]⍳ids←EIS ids
      :If ∨/mask←rows>1⊃⍴DATA
          ('IDs not found: ',⍕mask/ids)⎕SIGNAL 999
      :EndIf
    ∇
    ∇ values←GetValues id;countries
    ⍝ Get list of values of a given id for all COUNTRIES
      (values countries)←DATA[GetRows id;VALUE COUNTRY]
      values←(values,MISSVAL)[countries⍳COUNTRIES]
    ∇
    ∇ values←GDPtoDollar id;gdp
      values←(GetValues GetRows id)×(GetValues GetRows'GDP')
    ∇
    ∇ values←ratio RatioToPeople id
      values←((GetValues GetRows id)÷ratio)×(GetValue GetRows'POPULATION')
    ∇
    ∇ captions←GetCaptions ids
      captions←{⍺,' (',⍵,')'}/DATA[GetRows ids;TITLE UNIT]
      :If 1≥|≡ids ⋄ captions←⊃captions ⋄ :EndIf
    ∇
    ∇ infos←GetInfos ids
      infos←DATA[GetRows ids;INFO]
      infos←¯1↓∊(⊂3 1 1/' • '),infos,[1.5]⎕UCS 10
    ∇
    ∇ tags←ids GetTags values;titles
      titles←DATA[GetRows ids;TITLE]
      tags←titles{,/(⊂' ',⍺,'='),[1.5]⍕¨⍵}¨values
      tags←,/COUNTRIES,':',⍉↑tags
    ∇

    ∇ sp←InitSharpPlot(log notes)
      sp←⎕NEW Causeway.SharpPlot
      sp.SetMargins 20(40+10×notes)60 30 ⍝ Top Bottom Left Right
      sp.MissingValue←MISSVAL
      log←2↑log
      sp.XAxisStyle←+/(1,log[1])/Causeway.XAxisStyles.(GridLines LogScale)
      sp.YAxisStyle←+/(1,log[2])/Causeway.YAxisStyles.(GridLines LogScale)
      sp.SetColors⊂System.Drawing.Color.(Navy Maroon Green DarkOrange Purple Teal)
      sp.SetFootnoteFont'Times' 5 System.Drawing.FontStyle.Regular System.Drawing.Color.Green
    ∇

    ∇ sp←log Scatter(idx idy);x;y;rowx;rowy;rowz
      (x y)←GetValues¨(idx idy)
      sp←InitSharpPlot log 2
      sp.SetMarkers Causeway.Marker.Plus
      sp.ScatterPlotStyle←Causeway.ScatterPlotStyles.(HaloMarkers)
      sp.(XCaption YCaption)←GetCaptions idx idy
      sp.Footnote←GetInfos idx idy
      sp.AddTips⊂idx idy GetTags x y
      sp.DrawScatterPlot y x
    ∇

    ∇ sp←log Bubble(idx idy idz);x;y;z;rowx;rowy;rowz
      (x y z)←GetValues¨(idx idy idz)
      sp←InitSharpPlot log 3
      sp.MarginRight+←50
      sp.SetMarkers Causeway.Marker.Bullet
      sp.SetMarkerScales 3
      sp.BubbleChartStyle←Causeway.BubbleChartStyles.(ScaleBar+HaloMarkers)
      sp.(XCaption YCaption ZCaption)←GetCaptions idx idy idz
      sp.Footnote←GetInfos idx idy idz
      sp.SplitBy(1+0>z)('Positive' 'Negative')
      sp.AddTips⊂2⍴⊂idx idy idz GetTags x y z  ⍝ tips for the two series to come
      sp.DrawBubbleChart y x z
    ∇

    ∇ sp←opt Contour(idx idy idz);x;y;z;rowx;rowy;rowz;type;col;log;ztick;flex;dens
    ⍝ Draw multiple contours (there may be several idz's)
      (log ztick flex dens)←opt
      (x y)←GetValues¨(idx idy)
      z←GetValues¨idz←,EIS idz
      sp←InitSharpPlot log(2+≢idz)
      sp.SetMarkers Causeway.Marker.Plus
      sp.ContourPlotStyle←Causeway.ContourPlotStyles.(HaloMarkers+Curves)
      sp.SetOrderOfFit 0 0   ⍝ not a polynomial fit
      sp.Flexibility←flex  ⍝ smooth trend surface rather than polynomial fit
      sp.MeshDensity←dens
      sp.(XCaption YCaption)←GetCaptions idx idy
      sp.Footnote←GetInfos idx idy,idz
      sp.SetKeyText⊂GetCaptions idz
      col←(≢z)⍴sp.GetColors  ⍝ user colors for contour lines
      sp.AddTips⊂(idx idy,idz)GetTags(x y,z)  ⍝ single tip for all series
      ztick←(≢z)↑ztick
      :For z ztick col :InEach z ztick col
          sp.SetZTickMarks ztick~0  ⍝ 0 resets to default
          sp.SetContourStyle col
          sp.DrawContourPlot x y z
      :EndFor
      sp.ScatterPlotStyle←Causeway.ScatterPlotStyles.HaloMarkers
      sp.SetColors System.Drawing.Color.Black  ⍝ blacken all markers
      sp.DrawScatterPlot y x
    ∇




:EndNamespace